@@ -157,6 +157,11 @@ class TemplatedNaturalLanguageGenerator(NaturalLanguageGenerator):
             for key in keys_to_interpolate:
                 if key in template:
                     template[key] = interpolate(template[key], template_vars)
+
+        # Inject the language attribute into the template based on the current session_language
+        if template.get('custom'):
+            template['custom']['language'] = template_vars.get('session_language') or 'English'
+            
         return template
 
     def iterateNestedDict(self, custom: Dict[Text, Any], template_vars: Dict[Text, Any]):
