@@ -6,11 +6,18 @@ import os
 import copy
 from types import LambdaType
 from typing import Any, Dict, List, Optional, Text, Tuple
-
+from rasa.core.utils import AvailableEndpoints
+import aiohttp
+import requests
+from os import environ
 import numpy as np
 import time
 import re
+import string
 from text2digits import text2digits
+from metrics.constants import APP_NAV
+
+
 t2d = text2digits.Text2Digits()
 
 from metrics.constants import ACTION_PREDICT_EXECUTE_TIME
@@ -27,7 +34,7 @@ from rasa.core.constants import (
     USER_INTENT_RESTART,
     UTTER_PREFIX,
     USER_INTENT_BACK,
-    USER_INTENT_OUT_OF_SCOPE,
+    USER_INTENT_OUT_OF_SCOPE
 )
 from rasa.core.domain import Domain
 from rasa.core.events import (
@@ -53,25 +60,30 @@ from rasa.core.tracker_store import TrackerStore
 from rasa.core.trackers import DialogueStateTracker, EventVerbosity
 from rasa.utils.endpoints import EndpointConfig
 from rasa.utils.timer import timeit
+from rasa.utils.redactor import Redactor
 
 logger = logging.getLogger(__name__)
 
-
 MAX_NUMBER_OF_PREDICTIONS = int(os.environ.get("MAX_NUMBER_OF_PREDICTIONS", "10"))
+MAX_PARTIALS_PROCESSED = int(os.environ.get("MAX_PARTIALS_PROCESSED", 2))
+FC_ENDPOINT = os.environ.get('FC_ENDPOINT')
+FC_ACCOUNT_ID = os.environ.get('FC_ACCOUNT_ID')
+FC_ACCOUNT_TOKEN = os.environ.get('FC_ACCOUNT_TOKEN')
+redactor = Redactor()
 
 
 class MessageProcessor:
     def __init__(
-        self,
-        interpreter: NaturalLanguageInterpreter,
-        policy_ensemble: PolicyEnsemble,
-        domain: Domain,
-        tracker_store: TrackerStore,
-        generator: NaturalLanguageGenerator,
-        action_endpoint: Optional[EndpointConfig] = None,
-        max_number_of_predictions: int = MAX_NUMBER_OF_PREDICTIONS,
-        message_preprocessor: Optional[LambdaType] = None,
-        on_circuit_break: Optional[LambdaType] = None,
+            self,
+            interpreter: NaturalLanguageInterpreter,
+            policy_ensemble: PolicyEnsemble,
+            domain: Domain,
+            tracker_store: TrackerStore,
+            generator: NaturalLanguageGenerator,
+            action_endpoint: Optional[EndpointConfig] = None,
+            max_number_of_predictions: int = MAX_NUMBER_OF_PREDICTIONS,
+            message_preprocessor: Optional[LambdaType] = None,
+            on_circuit_break: Optional[LambdaType] = None,
     ):
         self.interpreter = interpreter
         self.nlg = generator
@@ -84,7 +96,7 @@ class MessageProcessor:
         self.action_endpoint = action_endpoint
 
     async def handle_message(
-        self, message: UserMessage
+            self, message: UserMessage
     ) -> Optional[List[Dict[Text, Any]]]:
         """Handle a single message with this processor."""
 
@@ -147,7 +159,7 @@ class MessageProcessor:
         }
 
     async def log_message(
-        self, message: UserMessage, should_save_tracker: bool = True
+            self, message: UserMessage, should_save_tracker: bool = True
     ) -> Optional[DialogueStateTracker]:
         """Log `message` on tracker belonging to the message's conversation_id.
 
@@ -229,7 +241,7 @@ class MessageProcessor:
 
     @staticmethod
     def _is_reminder_still_valid(
-        tracker: DialogueStateTracker, reminder_event: ReminderScheduled
+            tracker: DialogueStateTracker, reminder_event: ReminderScheduled
     ) -> bool:
         """Check if the conversation has been restarted after reminder."""
 
@@ -240,7 +252,7 @@ class MessageProcessor:
 
     @staticmethod
     def _has_message_after_reminder(
-        tracker: DialogueStateTracker, reminder_event: ReminderScheduled
+            tracker: DialogueStateTracker, reminder_event: ReminderScheduled
     ) -> bool:
         """Check if the user sent a message after the reminder."""
 
@@ -350,6 +362,16 @@ class MessageProcessor:
         else:
             return 0.0
 
+    def get_group(self, tracker: DialogueStateTracker):
+        last_bot_utter_template = tracker.latest_bot_utterance.metadata.get("template_name")
+        if last_bot_utter_template:
+            bot_responses = self.domain.templates.get(last_bot_utter_template, [])
+            for bot_response in bot_responses:
+                if bot_response.get("custom") and bot_response.get("custom").get("group"):
+                    return bot_response.get("custom").get("group")
+                elif bot_response.get("button") and bot_response.get("button").get("group"):
+                    return bot_response.get("button").get("group")
+
     def _log_unseen_features(self, parse_data: Dict[Text, Any]) -> None:
         """Check if the NLU interpreter picks up intents or entities that aren't recognized."""
 
@@ -384,6 +406,7 @@ class MessageProcessor:
     def _get_action(self, action_name):
         return self.domain.action_for_name(action_name, self.action_endpoint)
 
+
     async def _parse_message(self, message, tracker: DialogueStateTracker = None):
         # for testing - you can short-cut the NLU part with a message
         # in the format /intent{"entity1": val1, "entity2": val2}
@@ -397,17 +420,75 @@ class MessageProcessor:
                 message.text, message.message_id, tracker
             )
 
-        logger.debug(
+        red_parse_data = redactor.redact_event(parse_data)
+        red_parse_data['text'] = self.convert_messge_to_digits(message.text)
+        red_parse_data = {'parse_data': red_parse_data}
+        bot_msg_data = {}
+        if tracker and tracker.latest_bot_utterance:
+            bot_msg_data = tracker.latest_bot_utterance.as_dict()
+        should_redact = bot_msg_data.get('privacy_mode', False)
+        if should_redact: 
+            red_parse_data = redactor.redact_event_complete(red_parse_data)
+        else:
+            red_parse_data = redactor.redact_event(red_parse_data)
+
+        logger.info(
             "[{}] - Received user message '{}' with intent '{}' "
             "and entities '{}'".format(
-                message.sender_id, message.text, parse_data["intent"], parse_data["entities"]
-            )
-        )
+                message.sender_id, red_parse_data['parse_data']['text'], red_parse_data['parse_data']["intent"], red_parse_data['parse_data']["entities"]
+            ))
+        logger.debug("[{}] - Received parse_data '{}'".format(message.sender_id, red_parse_data))
+        confidence_threshold = self.get_nlu_threshold()
+        if type(message.metadata) is not dict:
+            if type(message.metadata) is str:
+                logger.warning("metadata of type 'str' changing to empty dictionary")
+            message.metadata = {}
+        message_type = message.metadata.get("message_type")
+        intent_name = parse_data["intent"]["name"]
+        partials = message.metadata.get("partials", [])
+        original_text = self.convert_messge_to_digits(message.text)
+        message.metadata["original_text"] = original_text
+        if message_type == "SPOKEN" and len(partials) > 0 and (
+                parse_data["intent"]["confidence"] < confidence_threshold or
+                (intent_name == "action_handle_error" and message.text) or
+                (intent_name == "out_of_scope" and message.text) or
+                not intent_name or
+                re.match('action_\w{0,50}_fallback', intent_name) or
+                re.match('action_\w{0,50}_followup', intent_name)):
+            logger.debug("Entering partials strategy with parse_data: {}".format(red_parse_data))
+
+            partials = partials[::-1]  # reverse list
+            if len(partials) > MAX_PARTIALS_PROCESSED:
+                partials = partials[:MAX_PARTIALS_PROCESSED]
+
+            logger.info(
+                "[{}] - Beginning partials strategy with '{}' as partials".format(message.sender_id,
+                                                                                  self.convert_messge_to_digits(str(partials))))
+            for partial in partials:
+                partials_parse_data = await self.interpreter.parse(partial, message.message_id, tracker)
+                if partials_parse_data["intent"]["name"] and partials_parse_data["intent"]["confidence"] > confidence_threshold and (
+                        partials_parse_data["intent"]["name"] != "action_handle_error" or
+                        partials_parse_data["intent"]["name"] != "out_of_scope"):
+                    
+                    red_parse_data_partials = redactor.redact_event_complete(partials_parse_data)
+                    #red_parse_data_partials['text'], red_parse_data_partials["intent"], red_parse_data["entities"]
+                    # logger.info(
+                    #     "[{}] - Received user message '{}' with intent '{}' "
+                    #     "and entities '{}' from partial".format(
+                    #         message.sender_id, red_parse_data_partials['text'], red_parse_data_partials["intent"], red_parse_data_partials["entities"]
+                    #     ))
+                    #self._log_unseen_features(parse_data)
+                    return partials_parse_data
 
         self._log_unseen_features(parse_data)
-
         return parse_data
 
+    def get_last_bot_msg(self, tracker: DialogueStateTracker):
+        for event in reversed(tracker.events):
+            if (event.__class__.__name__ == "BotUttered" and event.data and event.data['custom'] and event.data['custom']['type'] != 'send_sip_info'):
+                return event
+        return None
+
     def create_metadata(
             self, message: UserMessage, tracker: DialogueStateTracker
     ) -> Dict:
@@ -419,11 +500,6 @@ class MessageProcessor:
             metadata["type"] = metadata.get("replyType")
         else:
             metadata["type"] = "text"
-        
-        if metadata and metadata.get("message_type"):
-            metadata["message_type"] = metadata.get("message_type")
-        else:
-            metadata["message_type"] = None
 
         if metadata and metadata.get("error_message"):
             metadata["error"] = True
@@ -434,7 +510,7 @@ class MessageProcessor:
 
         if tracker.active_form.get("name"):
             metadata["active_form"] = tracker.active_form.get("name")
-        
+
         if handoff_active:
             metadata["handoff_active"] = handoff_active
 
@@ -449,11 +525,22 @@ class MessageProcessor:
             elif requested_slot == "id":
                 metadata["type"] = "username"
 
-        if tracker.latest_bot_utterance.metadata and "template_name" in tracker.latest_bot_utterance.metadata:
+        if tracker.latest_bot_utterance and tracker.latest_bot_utterance.metadata and "template_name" in tracker.latest_bot_utterance.metadata:
             metadata["template_name"] = tracker.latest_bot_utterance.metadata.get("template_name")
 
+        last_bot_msg = self.get_last_bot_msg(tracker)
+        if last_bot_msg:
+            if last_bot_msg.metadata and "template_name" in last_bot_msg.metadata:
+                metadata["template_name"] = last_bot_msg.metadata.get("template_name")
+
+            if last_bot_msg.data and "custom" in last_bot_msg.data:
+                custom = last_bot_msg.data["custom"]
+                if "grammar_file" in custom:
+                    metadata["grammar_file"] = custom["grammar_file"]
+
         metadata["asr_threshold"] = self.get_asr_threshold(tracker)
         metadata["nlu_threshold"] = self.get_nlu_threshold()
+        metadata["group"] = self.get_group(tracker)
 
         return metadata
 
@@ -463,38 +550,91 @@ class MessageProcessor:
         else:
             logger.info(f"[{tracker.sender_id}] - User msg: {message}")
 
+    # COH requirement for redacting pci data that may be given for the wrong prompt
+    def convert_messge_to_digits(self, message):
+        pci_msg = message
+        if '/greet' not in pci_msg and '/disconnect' not in pci_msg and '/restart' not in pci_msg:
+            if len(pci_msg) > 0 and pci_msg[-1] in string.punctuation and (pci_msg[-1] not in ['#', '*']):
+                pci_msg = pci_msg[:-1]
+            pci_msg = pci_msg.replace(',', ' ').replace('for ', 'four ').replace('to ', 'two ').replace('too ', 'two ').replace('won ', 'one ').replace('hate ', 'eight ').replace('ate ', 'eight ')
+            pci_msg_digits = t2d.convert(pci_msg)
+            if len(re.findall("\d{12,}", pci_msg_digits)) > 0:
+                pci_msg_digits = re.sub(r'(?<=\d)+ (?=\d)+', '', pci_msg_digits)
+                return pci_msg_digits
+            else:
+                pci_msg = re.sub(r'(?<=\d)+ (?=\d)+', '', pci_msg)  # strips any white space between digits
+                return pci_msg
+        return message
+
+    def convert_pci_entities(self, message):
+        pci_msg = message
+        num_list = []
+        for i in pci_msg:
+            if i['entity'] == 'number':
+                num_list.append(i)
+            elif len(num_list) <= 11:
+                num_list = []
+
+        for e in pci_msg:
+            e['value'] = re.sub(r"(?<!\d)(\d{12,})(?!\d)", '****************', str(e['value']))
+        if len(num_list) > 11:
+            for e in pci_msg:
+                if e['entity'] == 'number' and filter(lambda x: x['start'] == (e['start'] and x['end'] == e['end']),
+                                                      num_list):
+                    e['value'] = "****"
+                    e['text'] = "****"
+        return pci_msg
+
     async def _handle_message_with_tracker(
-        self, message: UserMessage, tracker: DialogueStateTracker
+            self, message: UserMessage, tracker: DialogueStateTracker
     ) -> None:
 
         if message.parse_data:
             parse_data = message.parse_data
         else:
             parse_data = await self._parse_message(message, tracker)
-        
-        message.metadata = self.create_metadata(message, tracker)
-        
-        isDTMF = False
-        if message.metadata['message_type']:
-            isDTMF = message.metadata['message_type'] == 'DTMF'
-        
-        redacted_text = message.text
-        redacted_entities =  copy.deepcopy(parse_data['entities'])
 
-        if not isDTMF and '/greet' not in redacted_text:
+        if parse_data['text']:
+            parse_data['text'] = self.convert_messge_to_digits(parse_data['text'])
+            
+        if parse_data.get('metadata') and parse_data.get('metadata').get('original_text'):
+            parse_data['metadata']['original_text'] = self.convert_messge_to_digits(parse_data['text'])
+
+        if parse_data.get('metadata') and parse_data.get('metadata').get('partials'):
+            new_partial = []
+            for partial in parse_data.get('metadata').get('partials'):
+                  new_partial.append(convert_messge_to_digits(partial))
+            parse_data['metadata']['partials'] = new_partial
+            
+        bot_msg_data = tracker.latest_bot_utterance.as_dict()
+        should_redact = bot_msg_data.get('privacy_mode')
+        redacted_text = self.convert_messge_to_digits(message.text)
+        original_text = message.metadata.get('original_text')
+        partials = message.metadata.get('partials')
+        redacted_entities = copy.deepcopy(parse_data['entities'])  # NOTE: They will not be redacted at this point but this is needed for tracker-consumers
+        message.metadata['should_redact'] = bool(should_redact)
+        if should_redact:
             redacted_text = UserUttered.redact_msg(redacted_text)
-            if redacted_text != message.text:
-                for e in redacted_entities:
-                    e['value'] = UserUttered.redact_msg(str(e['value']))
+            if parse_data['text']:
+                parse_data['text'] = UserUttered.redact_msg(self.convert_messge_to_digits(parse_data['text']))
+            if original_text:
+                converted_original_text = self.convert_messge_to_digits(original_text)
+                message.metadata['original_text'] = UserUttered.redact_msg(converted_original_text)
+            if partials and len(partials) > 0:
+                converted_partials = []
+                for partial in partials:
+                    if partial and len(partial) > 0:
+                        converted_partial = self.convert_messge_to_digits(partial)
+                        converted_partials.append(UserUttered.redact_msg(converted_partial))
+                message.metadata['partials'] = converted_partials
 
         missed_intent = not self._is_nlu_threshold_passed(parse_data, tracker)
-        
-        self.log_user_msg(redacted_text, message.metadata, tracker)
+        message.metadata = self.create_metadata(message, tracker)
+        #self.log_user_msg(redacted_text, message.metadata, tracker)
 
         # don't ever directly mutate the tracker
         # - instead pass its events to log
-        tracker.update(
-            UserUttered(
+        user_uttered = UserUttered(
                 message.text,
                 parse_data["intent"],
                 parse_data["entities"],
@@ -507,13 +647,19 @@ class MessageProcessor:
                 metadata=message.metadata,
                 text_redacted=redacted_text,
                 entities_redacted=redacted_entities
-            ),
+            )
+
+        
+        tracker.update(
+            user_uttered,
             self.domain,
         )
 
         if parse_data["entities"]:
             self._log_slots(tracker)
 
+        self.log_appNav(tracker.sender_id, user_uttered)
+        
         logger.debug(
             "[{}] - Logged UserUtterance - "
             "tracker now has {} events".format(tracker.sender_id, len(tracker.events))
@@ -522,13 +668,67 @@ class MessageProcessor:
     @staticmethod
     def _should_handle_message(tracker: DialogueStateTracker):
         return (
-            not tracker.is_paused()
-            or tracker.latest_message.intent.get("name") == USER_INTENT_RESTART
+                not tracker.is_paused()
+                or tracker.latest_message.intent.get("name") == USER_INTENT_RESTART
         )
 
+    async def send_app_nav(self, conversation_id, app_nav):
+        method = 'POST'
+        payload = {
+            'appNavs': [app_nav]
+        }
+        endpoint = f"{FC_ENDPOINT}/Accounts/{FC_ACCOUNT_ID}/Calls/{conversation_id}/AppNavs"
+        requests.post(endpoint, json=payload, auth=(FC_ACCOUNT_ID,FC_ACCOUNT_ID))
+        return None
+
+
+    def log_appNav(self, sender_id: str, message: UserUttered):
+        try:
+            if (message.text == '/disconnect'):
+                app_nav = {
+                    'key': '{}_disconnect'.format(message._metadata['template_name']),
+                    'value:': 'disconnect'
+                }
+                logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
+                asyncio.create_task(self.send_app_nav(sender_id, app_nav))
+                APP_NAV.labels(message._metadata['template_name'], 'disconnect').inc()
+
+            elif message._metadata and message._metadata.get('message_type', None) == 'SPOKEN':
+                if message.metadata.get('reporting_error_type', None):
+                    app_nav = {
+                    'key': '{}_nlu_{}'.format(message._metadata['template_name'], message._metadata['reporting_error_type']),
+                    'value:': 'NO_NTENT'
+                    }
+                    logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
+                    asyncio.create_task(self.send_app_nav(sender_id,app_nav))
+                    APP_NAV.labels(message._metadata['template_name'], message._metadata['reporting_error_type']).inc()
+                    
+                elif message.missed_intent and message.intent['confidence'] < message._metadata['nlu_threshold']:
+                    app_nav = {
+                    'key': '{}_nlu_no_match'.format(message._metadata['template_name']),
+                    'value:': 'NO_NTENT'
+                    }
+                    logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
+                    asyncio.create_task(self.send_app_nav(sender_id,app_nav))
+                    APP_NAV.labels(message._metadata['template_name'], 'no_match').inc()
+                    
+                elif message.intent['confidence'] > message._metadata['nlu_threshold']:
+                    app_nav = {
+                    'key': '{}_nlu_match'.format(message._metadata['template_name'] ),
+                    'value:': message.intent
+                    }
+                    logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
+                    asyncio.create_task(self.send_app_nav(sender_id,app_nav))
+                    APP_NAV.labels(message._metadata['template_name'], 'match').inc()
+                else:
+                    logger.debug("[{}] - APP NAV: Error logging app_nav: {}".format(sender_id, message))
+
+        except Exception as e:
+            logger.debug("APP NAV: Error logging app nav")
+
     # @timeit
     async def _predict_and_execute_next_action(
-        self, message: UserMessage, tracker: DialogueStateTracker
+            self, message: UserMessage, tracker: DialogueStateTracker
     ):
         # keep taking actions decided by the policy until it chooses to 'listen'
         should_predict_another_action = True
@@ -536,15 +736,15 @@ class MessageProcessor:
 
         def is_action_limit_reached():
             return (
-                num_predicted_actions == self.max_number_of_predictions
-                and should_predict_another_action
+                    num_predicted_actions == self.max_number_of_predictions
+                    and should_predict_another_action
             )
 
         # action loop. predicts actions until we hit action listen
         while (
-            should_predict_another_action
-            and self._should_handle_message(tracker)
-            and num_predicted_actions < self.max_number_of_predictions
+                should_predict_another_action
+                and self._should_handle_message(tracker)
+                and num_predicted_actions < self.max_number_of_predictions
         ):
             # this actually just calls the policy's method by the same name
             action, policy, confidence = self.predict_next_action(tracker)
@@ -575,9 +775,9 @@ class MessageProcessor:
     @staticmethod
     # @timeit
     async def _send_bot_messages(
-        events: List[Event],
-        tracker: DialogueStateTracker,
-        output_channel: OutputChannel,
+            events: List[Event],
+            tracker: DialogueStateTracker,
+            output_channel: OutputChannel,
     ) -> None:
         """Send all the bot messages that are logged in the events array."""
         for e in events:
@@ -590,11 +790,11 @@ class MessageProcessor:
             await output_channel.send_response(tracker.sender_id, e.message())
 
     async def _schedule_reminders(
-        self,
-        events: List[Event],
-        tracker: DialogueStateTracker,
-        output_channel: OutputChannel,
-        nlg: NaturalLanguageGenerator,
+            self,
+            events: List[Event],
+            tracker: DialogueStateTracker,
+            output_channel: OutputChannel,
+            nlg: NaturalLanguageGenerator,
     ) -> None:
         """Uses the scheduler to time a job to trigger the passed reminder.
 
@@ -612,24 +812,24 @@ class MessageProcessor:
                 id=e.name,
                 replace_existing=True,
                 name=(
-                    str(e.action_name)
-                    + ACTION_NAME_SENDER_ID_CONNECTOR_STR
-                    + tracker.sender_id
+                        str(e.action_name)
+                        + ACTION_NAME_SENDER_ID_CONNECTOR_STR
+                        + tracker.sender_id
                 ),
             )
 
     @staticmethod
     async def _cancel_reminders(
-        events: List[Event], tracker: DialogueStateTracker
+            events: List[Event], tracker: DialogueStateTracker
     ) -> None:
         """Cancel reminders by action_name"""
         # All Reminders with the same action name will be cancelled
         for e in events:
             if isinstance(e, ReminderCancelled):
                 name_to_check = (
-                    str(e.action_name)
-                    + ACTION_NAME_SENDER_ID_CONNECTOR_STR
-                    + tracker.sender_id
+                        str(e.action_name)
+                        + ACTION_NAME_SENDER_ID_CONNECTOR_STR
+                        + tracker.sender_id
                 )
                 scheduler = await jobs.scheduler()
                 for j in scheduler.get_jobs():
@@ -638,7 +838,7 @@ class MessageProcessor:
 
     # @timeit
     async def _run_action(
-        self, action, tracker, output_channel, nlg, policy=None, confidence=None
+            self, action, tracker, output_channel, nlg, policy=None, confidence=None
     ):
         # events and return values are used to update
         # the tracker state after an action has been taken
@@ -654,13 +854,13 @@ class MessageProcessor:
                 "Bot will continue, but the actions events are lost. "
                 "Please check the logs of your action server for "
                 "more information.".format(tracker.sender_id, action.name())
-            , exc_info=True)
+                , exc_info=True)
             # logger.error(e, exc_info=True)
             events = []
 
         self._log_action_on_tracker(tracker, action.name(), events, policy, confidence)
         if action.name() != ACTION_LISTEN_NAME and not action.name().startswith(
-            UTTER_PREFIX
+                UTTER_PREFIX
         ):
             self._log_slots(tracker)
 
@@ -734,7 +934,7 @@ class MessageProcessor:
         self.tracker_store.save(tracker)
 
     def _prob_array_for_action(
-        self, action_name: Text
+            self, action_name: Text
     ) -> Tuple[Optional[List[float]], None]:
         idx = self.domain.index_for_action(action_name)
         if idx is not None:
@@ -745,7 +945,7 @@ class MessageProcessor:
             return None, None
 
     def _get_next_action_probabilities(
-        self, tracker: DialogueStateTracker
+            self, tracker: DialogueStateTracker
     ) -> Tuple[Optional[List[float]], Optional[Text]]:
         """Collect predictions from ensemble and return action and predictions.
         """
