@@ -1,7 +1,8 @@
 import asyncio
 import time
-
+import json
 import logging
+import warnings
 import os
 
 import aiohttp
@@ -66,6 +67,9 @@ class DucklingHTTPExtractor(EntityExtractor):
         # timezone like Europe/Berlin
         # if not set the default timezone of Duckling is going to be used
         "timezone": None,
+        # Timeout for receiving response from http url of the running duckling server
+        # if not set the default timeout of duckling http url is set to 30 seconds.
+        "timeout": 30,
     }
 
     def __init__(
@@ -74,8 +78,9 @@ class DucklingHTTPExtractor(EntityExtractor):
         language: Optional[Text] = None,
     ) -> None:
 
-        super(DucklingHTTPExtractor, self).__init__(component_config)
+        super().__init__(component_config)
         self.language = language
+        self.session = aiohttp.ClientSession()
 
     @classmethod
     def create(
@@ -101,40 +106,43 @@ class DucklingHTTPExtractor(EntityExtractor):
         return self.component_config.get("url")
 
     def _payload(self, text, reference_time):
+        dimensions = self.component_config["dimensions"]
         return {
             "text": text,
             "locale": self._locale(),
             "tz": self.component_config.get("timezone"),
+            "dims": json.dumps(dimensions),
             "reftime": reference_time,
         }
 
     async def _duckling_parse(self, text, reference_time):
         """Sends the request to the duckling server and parses the result."""
-        start = time.time()
+        start = time.perf_counter()
         status = 500
         try:
             payload = self._payload(text, reference_time)
             headers = {
                 "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"
             }
-            async with aiohttp.ClientSession() as session:
-                async with session.post(self._url() + "/parse", data=payload, headers=headers, timeout=3) as response:
-                    if response.status == 200:
-                        return await response.json()
-                    else:
-                        logger.error(
-                            "Failed to get a proper response from remote "
-                            "duckling. Status Code: {}. Response: {}"
-                            "".format(response.status, response.text)
-                        )
-                        return []
+
+            async with self.session.post(self._url() + "/parse", data=payload, headers=headers, timeout=30) as response:
+                status = response.status
+                if response.status == 200:
+                    return await response.json()
+                else:
+                    logger.error(
+                        "Failed to get a proper response from remote "
+                        "duckling. Status Code: {}. Response: {}"
+                        "".format(response.status, response.text)
+                    )
+                    return []
         except (
                 asyncio.CancelledError,
                 asyncio.TimeoutError
         ) as e:
             logger.error(
                 "Failed to connect to duckling http server. Make sure "
-                "the duckling server is running and the proper host "
+                "the duckling server is running/healthy/not stale and the proper host "
                 "and port are set in the configuration. More "
                 "information on how to run the server can be found on "
                 "github: "
@@ -143,8 +151,8 @@ class DucklingHTTPExtractor(EntityExtractor):
             )
             return []
         finally:
-            duration = time.time() - start
-            logger.info(f"Duckling request: {duration}")
+            duration = time.perf_counter() - start
+            logger.info(f"Duckling request: {round(duration, 2)}s")
             OUTGOING_REQUEST_LATENCY_SEC.labels(self._url() + "/parse", status, "POST").observe(duration)
 
     @staticmethod
@@ -175,7 +183,7 @@ class DucklingHTTPExtractor(EntityExtractor):
             )
         else:
             extracted = []
-            logger.warning(
+            warnings.warn(
                 "Duckling HTTP component in pipeline, but no "
                 "`url` configuration in the config "
                 "file nor is `RASA_DUCKLING_HTTP_URL` "
@@ -194,7 +202,7 @@ class DucklingHTTPExtractor(EntityExtractor):
         model_dir: Text = None,
         model_metadata: Optional[Metadata] = None,
         cached_component: Optional["DucklingHTTPExtractor"] = None,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> "DucklingHTTPExtractor":
 
         language = model_metadata.get("language") if model_metadata else None
