@@ -1,3 +1,4 @@
+import asyncio
 import logging
 import os
 
@@ -62,7 +63,7 @@ def concat_url(base: Text, subpath: Optional[Text]) -> Text:
     return url + subpath
 
 
-class EndpointConfig(object):
+class EndpointConfig:
     """Configuration for an external HTTP endpoint."""
 
     def __init__(
@@ -73,7 +74,7 @@ class EndpointConfig(object):
         basic_auth: Dict[Text, Text] = None,
         token: Optional[Text] = None,
         token_name: Text = "token",
-        **kwargs
+        **kwargs,
     ):
         self.url = url
         self.params = params if params else {}
@@ -93,9 +94,26 @@ class EndpointConfig(object):
         else:
             auth = None
 
+        async def on_request_start(session, trace_config_ctx, params):
+            trace_config_ctx.start = asyncio.get_event_loop().time()
+
+        async def on_request_end(session, trace_config_ctx, params):
+            if trace_config_ctx.trace_request_ctx:
+                elapsed = round(asyncio.get_event_loop().time() - trace_config_ctx.start, 2)
+                logger.info("[{}] - Request ({}) took {}s".format(
+                    trace_config_ctx.trace_request_ctx.get("sender_id", "NA"),
+                    trace_config_ctx.trace_request_ctx.get("url", "NA"),
+                    elapsed)
+                )
+
+        trace_config = aiohttp.TraceConfig()
+        trace_config.on_request_start.append(on_request_start)
+        trace_config.on_request_end.append(on_request_end)
+
         return aiohttp.ClientSession(
             headers=self.headers,
             auth=auth,
+            trace_configs=[trace_config],
             timeout=aiohttp.ClientTimeout(total=DEFAULT_REQUEST_TIMEOUT),
         )
 
@@ -118,7 +136,7 @@ class EndpointConfig(object):
         subpath: Optional[Text] = None,
         content_type: Optional[Text] = "application/json",
         return_method: Text = "json",
-        **kwargs: Any
+        **kwargs: Any,
     ):
         """Send a HTTP request to the endpoint.
 
@@ -135,13 +153,15 @@ class EndpointConfig(object):
             del kwargs["headers"]
 
         url = concat_url(self.url, subpath)
+        trace_request_ctx = {"sender_id": headers.get("Conversation-Id"), "url": url}
         async with self.session() as session:
             async with session.request(
                 method,
                 url,
                 headers=headers,
+                trace_request_ctx=trace_request_ctx,
                 params=self.combine_parameters(kwargs),
-                **kwargs
+                **kwargs,
             ) as resp:
                 if resp.status >= 400:
                     raise ClientResponseError(
@@ -161,7 +181,7 @@ class EndpointConfig(object):
             self.basic_auth,
             self.token,
             self.token_name,
-            **self.kwargs
+            **self.kwargs,
         )
 
     def __eq__(self, other):
@@ -186,7 +206,7 @@ class ClientResponseError(aiohttp.ClientError):
         self.status = status
         self.message = message
         self.text = text
-        super().__init__("{}, {}, body='{}'".format(status, message, text))
+        super().__init__(f"{status}, {message}, body='{text}'")
 
 
 def bool_arg(request: Request, name: Text, default: bool = True) -> bool:
@@ -214,5 +234,5 @@ def float_arg(
     try:
         return float(str(arg))
     except (ValueError, TypeError):
-        logger.warning("Failed to convert '{}' to float.".format(arg))
+        logger.warning(f"Failed to convert '{arg}' to float.")
         return default
