@@ -1,7 +1,9 @@
 import os
+from typing import Callable
+from _pytest.pytester import RunResult
 
 
-def test_init(run):
+def test_init(run: Callable[..., RunResult]):
     run("init", "--no-prompt", "--quiet")
 
     assert os.path.exists("actions.py")
@@ -14,7 +16,7 @@ def test_init(run):
     assert os.path.exists("data/stories.md")
 
 
-def test_init_help(run):
+def test_init_help(run: Callable[..., RunResult]):
     output = run("init", "--help")
 
     assert (
