@@ -1,14 +1,21 @@
 import json
+from pathlib import Path
 
 import pytest
 from _pytest.tmpdir import TempdirFactory
 
+from rasa.core.constants import (
+    DEFAULT_KNOWLEDGE_BASE_ACTION,
+    SLOT_LISTED_ITEMS,
+    SLOT_LAST_OBJECT,
+    SLOT_LAST_OBJECT_TYPE,
+)
 from rasa.core import training, utils
 from rasa.core.domain import Domain, InvalidDomain
 from rasa.core.featurizers import MaxHistoryTrackerFeaturizer
 from rasa.core.slots import TextSlot, UnfeaturizedSlot
-from tests.core import utilities
 from tests.core.conftest import DEFAULT_DOMAIN_PATH_WITH_SLOTS, DEFAULT_STORIES_FILE
+from rasa.utils import io as io_utils
 
 
 async def test_create_train_data_no_history(default_domain):
@@ -166,10 +173,9 @@ def test_utter_templates():
     assert domain.random_template_for("utter_greet") == expected_template
 
 
-def test_custom_slot_type(tmpdir):
-    domain_path = utilities.write_text_to_file(
-        tmpdir,
-        "domain.yml",
+def test_custom_slot_type(tmpdir: Path):
+    domain_path = str(tmpdir / "domain.yml")
+    io_utils.write_text_file(
         """
        slots:
          custom:
@@ -181,6 +187,7 @@ def test_custom_slot_type(tmpdir):
 
        actions:
          - utter_greet """,
+        domain_path,
     )
     Domain.load(domain_path)
 
@@ -213,9 +220,8 @@ def test_custom_slot_type(tmpdir):
     ],
 )
 def test_domain_fails_on_unknown_custom_slot_type(tmpdir, domain_unkown_slot_type):
-    domain_path = utilities.write_text_to_file(
-        tmpdir, "domain.yml", domain_unkown_slot_type
-    )
+    domain_path = str(tmpdir / "domain.yml")
+    io_utils.write_text_file(domain_unkown_slot_type, domain_path)
     with pytest.raises(ValueError):
         Domain.load(domain_path)
 
@@ -535,3 +541,26 @@ def test_clean_domain():
     actual = Domain.from_dict(cleaned)
 
     assert hash(actual) == hash(expected)
+
+
+def test_add_knowledge_base_slots(default_domain):
+    import copy
+
+    # don't modify default domain as it is used in other tests
+    test_domain = copy.deepcopy(default_domain)
+
+    test_domain.action_names.append(DEFAULT_KNOWLEDGE_BASE_ACTION)
+
+    slot_names = [s.name for s in test_domain.slots]
+
+    assert SLOT_LISTED_ITEMS not in slot_names
+    assert SLOT_LAST_OBJECT not in slot_names
+    assert SLOT_LAST_OBJECT_TYPE not in slot_names
+
+    test_domain.add_knowledge_base_slots()
+
+    slot_names = [s.name for s in test_domain.slots]
+
+    assert SLOT_LISTED_ITEMS in slot_names
+    assert SLOT_LAST_OBJECT in slot_names
+    assert SLOT_LAST_OBJECT_TYPE in slot_names
