@@ -2,13 +2,23 @@ import os
 
 import json
 from collections import Counter
+from typing import Text, Dict
 
 import numpy as np
+import pytest
 
 from rasa.core import training
 from rasa.core.interpreter import RegexInterpreter
-from rasa.core.training.dsl import StoryFileReader
-from rasa.core.events import ActionExecuted, UserUttered
+from rasa.core.training.dsl import StoryFileReader, EndToEndReader
+from rasa.core.domain import Domain
+from rasa.core.trackers import DialogueStateTracker
+from rasa.core.events import (
+    UserUttered,
+    ActionExecuted,
+    ActionExecutionRejected,
+    Form,
+    FormValidation,
+)
 from rasa.core.training.structures import Story
 from rasa.core.featurizers import (
     MaxHistoryTrackerFeaturizer,
@@ -115,6 +125,80 @@ async def test_persist_and_read_test_story(tmpdir, default_domain):
         existing_stories.discard(story_str)
 
 
+async def test_persist_form_story(tmpdir):
+    domain = Domain.load("data/test_domains/form.yml")
+
+    tracker = DialogueStateTracker("", domain.slots)
+
+    story = (
+        "* greet\n"
+        "    - utter_greet\n"
+        "* start_form\n"
+        "    - some_form\n"
+        '    - form{"name": "some_form"}\n'
+        "* default\n"
+        "    - utter_default\n"
+        "    - some_form\n"
+        "* stop\n"
+        "    - utter_ask_continue\n"
+        "* affirm\n"
+        "    - some_form\n"
+        "* stop\n"
+        "    - utter_ask_continue\n"
+        "    - action_listen\n"
+        "* form: inform\n"
+        "    - some_form\n"
+        '    - form{"name": null}\n'
+        "* goodbye\n"
+        "    - utter_goodbye\n"
+    )
+
+    # simulate talking to the form
+    events = [
+        UserUttered(intent={"name": "greet"}),
+        ActionExecuted("utter_greet"),
+        ActionExecuted("action_listen"),
+        # start the form
+        UserUttered(intent={"name": "start_form"}),
+        ActionExecuted("some_form"),
+        Form("some_form"),
+        ActionExecuted("action_listen"),
+        # out of form input
+        UserUttered(intent={"name": "default"}),
+        ActionExecutionRejected("some_form"),
+        ActionExecuted("utter_default"),
+        ActionExecuted("some_form"),
+        ActionExecuted("action_listen"),
+        # out of form input
+        UserUttered(intent={"name": "stop"}),
+        ActionExecutionRejected("some_form"),
+        ActionExecuted("utter_ask_continue"),
+        ActionExecuted("action_listen"),
+        # out of form input but continue with the form
+        UserUttered(intent={"name": "affirm"}),
+        FormValidation(False),
+        ActionExecuted("some_form"),
+        ActionExecuted("action_listen"),
+        # out of form input
+        UserUttered(intent={"name": "stop"}),
+        ActionExecutionRejected("some_form"),
+        ActionExecuted("utter_ask_continue"),
+        ActionExecuted("action_listen"),
+        # form input
+        UserUttered(intent={"name": "inform"}),
+        FormValidation(True),
+        ActionExecuted("some_form"),
+        ActionExecuted("action_listen"),
+        Form(None),
+        UserUttered(intent={"name": "goodbye"}),
+        ActionExecuted("utter_goodbye"),
+        ActionExecuted("action_listen"),
+    ]
+    [tracker.update(e) for e in events]
+
+    assert story in tracker.export_stories()
+
+
 async def test_read_story_file_with_cycles(tmpdir, default_domain):
     graph = await training.extract_story_graph(
         "data/test_stories/stories_with_cycle.md", default_domain
@@ -125,7 +209,8 @@ async def test_read_story_file_with_cycles(tmpdir, default_domain):
     graph_without_cycles = graph.with_cycles_removed()
 
     assert graph.cyclic_edge_ids != set()
-    assert graph_without_cycles.cyclic_edge_ids == set()
+    # sorting removed_edges converting set converting it to list
+    assert graph_without_cycles.cyclic_edge_ids == list()
 
     assert len(graph.story_steps) == len(graph_without_cycles.story_steps) == 5
 
@@ -281,3 +366,123 @@ async def test_read_stories_with_multiline_comments(tmpdir, default_domain):
     assert len(story_steps[2].events) == 7
     assert story_steps[3].block_name == "say goodbye"
     assert len(story_steps[3].events) == 2
+
+
+@pytest.mark.parametrize(
+    "line, expected",
+    [
+        (" greet: hi", {"intent": "greet", "true_intent": "greet", "text": "hi"}),
+        (
+            " greet: /greet",
+            {
+                "intent": "greet",
+                "true_intent": "greet",
+                "text": "/greet",
+                "entities": [],
+            },
+        ),
+        (
+            'greet: /greet{"test": "test"}',
+            {
+                "intent": "greet",
+                "entities": [
+                    {"entity": "test", "start": 6, "end": 22, "value": "test"}
+                ],
+                "true_intent": "greet",
+                "text": '/greet{"test": "test"}',
+            },
+        ),
+        (
+            'greet{"test": "test"}: /greet{"test": "test"}',
+            {
+                "intent": "greet",
+                "entities": [
+                    {"entity": "test", "start": 6, "end": 22, "value": "test"}
+                ],
+                "true_intent": "greet",
+                "text": '/greet{"test": "test"}',
+            },
+        ),
+        (
+            "mood_great: [great](feeling)",
+            {
+                "intent": "mood_great",
+                "entities": [
+                    {"start": 0, "end": 5, "value": "great", "entity": "feeling"}
+                ],
+                "true_intent": "mood_great",
+                "text": "great",
+            },
+        ),
+        (
+            'form: greet{"test": "test"}: /greet{"test": "test"}',
+            {
+                "intent": "greet",
+                "entities": [
+                    {"end": 22, "entity": "test", "start": 6, "value": "test"}
+                ],
+                "true_intent": "greet",
+                "text": '/greet{"test": "test"}',
+            },
+        ),
+    ],
+)
+def test_e2e_parsing(line: Text, expected: Dict):
+    reader = EndToEndReader()
+    actual = reader._parse_item(line)
+
+    assert actual.as_dict() == expected
+
+
+@pytest.mark.parametrize(
+    "parse_data, expected_story_string",
+    [
+        (
+            {
+                "text": "/simple",
+                "parse_data": {
+                    "intent": {"confidence": 1.0, "name": "simple"},
+                    "entities": [
+                        {"start": 0, "end": 5, "value": "great", "entity": "feeling"}
+                    ],
+                },
+            },
+            "simple: /simple",
+        ),
+        (
+            {
+                "text": "great",
+                "parse_data": {
+                    "intent": {"confidence": 1.0, "name": "simple"},
+                    "entities": [
+                        {"start": 0, "end": 5, "value": "great", "entity": "feeling"}
+                    ],
+                },
+            },
+            "simple: [great](feeling)",
+        ),
+        (
+            {
+                "text": "great",
+                "parse_data": {
+                    "intent": {"confidence": 1.0, "name": "simple"},
+                    "entities": [],
+                },
+            },
+            "simple: great",
+        ),
+    ],
+)
+def test_user_uttered_to_e2e(parse_data: Dict, expected_story_string: Text):
+    event = UserUttered.from_story_string("user", parse_data)[0]
+
+    assert isinstance(event, UserUttered)
+    assert event.as_story_string(e2e=True) == expected_story_string
+
+
+@pytest.mark.parametrize("line", [" greet{: hi"])
+def test_invalid_end_to_end_format(line: Text):
+    reader = EndToEndReader()
+
+    with pytest.raises(ValueError):
+        _ = reader._parse_item(line)
