@@ -61,6 +61,7 @@ async def test_message_id_logging(default_processor: MessageProcessor):
 
 
 async def test_parsing(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
     message = UserMessage('/greet{"name": "boy"}')
     parsed = await default_processor._parse_message(message)
     assert parsed["intent"]["name"] == "greet"
@@ -68,6 +69,7 @@ async def test_parsing(default_processor: MessageProcessor):
 
 
 async def test_log_unseen_feature(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
     message = UserMessage('/dislike{"test_entity": "RASA"}')
     parsed = await default_processor._parse_message(message)
     with pytest.warns(UserWarning) as record:
@@ -84,6 +86,7 @@ async def test_log_unseen_feature(default_processor: MessageProcessor):
 
 
 async def test_default_intent_recognized(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
     message = UserMessage("/restart")
     parsed = await default_processor._parse_message(message)
     with pytest.warns(None) as record:
@@ -177,29 +180,95 @@ async def test_reminder_scheduled(
     )
     assert t.events[-1] == ActionExecuted("action_listen")
 
-async def test_redaction(default_processor: MessageProcessor):
-    message_1 = UserUttered.redact_msg("hello")
-    message_2 = UserUttered.redact_msg("7/12/1996")
-    message_3 = UserUttered.redact_msg("five dollars")
-    message_4 = UserUttered.redact_msg("5 dollars")
-    message_5 = UserUttered.redact_msg("5.50")
-    message_6 = UserUttered.redact_msg("1a2a3")
-    message_7 = UserUttered.redact_msg("one hundred")
-    message_8 = UserUttered.redact_msg("5 and eight")
-    message_9 = UserUttered.redact_msg("one thousand six hundred sixty six" )
-    message_10 = UserUttered.redact_msg("one thousand and six hundred and sixty six")
-    assert message_1 == "hello"
-    assert message_2 == "****"
-    assert message_3 == "**** dollars"
-    assert message_4 == "**** dollars"
-    assert message_5 == "****"
-    assert message_6 == "****"
-    assert message_7 == "****"
-    assert message_8 == "**** and ****"
-    assert message_9 == "****"
-    assert message_10 == "****"
+async def test_convert_messge_to_digits_1(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
 
+    message = UserMessage("one,one,one,one,one,one,one,one,one,one,one,one,one,one,one,one")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "1111111111111111"
 
+async def test_convert_messge_to_digits_2(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("1,2,3,4,5,6,7,8,9,0,9,8,7,6,5,4")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "1234567890987654"
+
+async def test_convert_messge_to_digits_3(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("make a one time payment")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "make a one time payment"
+
+async def test_convert_messge_to_digits_4(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("one,one,one,one,one one one,one,one,one,one,one,one,one,one,one.")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "1111111111111111"
+
+async def test_convert_messge_to_digits_5(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("one one one one one one one one one one")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "one one one one one one one one one one"
+
+async def test_convert_messge_to_digits_6(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("one one one one one one one one one one one one one")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "1111111111111"
+
+async def test_convert_messge_to_digits_7(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("my cc is one one one one one one one one one one one one one one one one")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "my cc is 1111111111111111"
+
+async def test_convert_messge_to_digits_8(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("my cc is won too hate for to ate one one one one one one one one one one")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "my cc is 1284281111111111"
+
+async def test_convert_messge_to_digits_greet(default_processor: MessageProcessor):
+    from rasa.core.trackers import DialogueStateTracker
+
+    message = UserMessage("/greet{guid: 'BE45375', channelUserId: '+17897897898'}")
+    tracker = DialogueStateTracker("1", [])
+    await default_processor._handle_message_with_tracker(message, tracker)
+    logged_event = tracker.events[-1]
+    assert logged_event.entities_redacted is not None
+    assert logged_event.text_redacted == "/greet{guid: 'BE45375', channelUserId: '+17897897898'}"
 
 async def test_reminder_aborted(
     default_channel: CollectingOutputChannel, default_processor: MessageProcessor
